##################################################################################
#
#   Kaivo Public Software License
#
#   Copyright (c) 2001, Kaivo, Inc.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions are met:
#
#       o Redistributions of source code must retain the above copyright notice,
#         this list of conditions and the following disclaimer.
#
#       o Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer
#         in the documentation and/or other materials provided with the
#         distribution.
#
#       o Neither Kaivo nor the names of its contributors may be used to endorse
#         or promote products derived from this software without specific prior
#         written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL KAIVO OR ITS CONTRIBUTORS BE LIABLE FOR 
#   ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
#   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##################################################################################

__doc__ = """A Zope Document Library"""
__version__ = '0.1a'

# Document Library by Casey Duncan (cduncan@kaivo.com)
# Kaivo, Inc. (http://www.kaivo.com)

import os
from string import strip, upper
import Globals
from Globals import DTMLFile
from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager
from OFS.DTMLMethod import DTMLMethod
from DateTime import DateTime
from Products.ZCatalog.Vocabulary import Vocabulary
from TopicIndex import TopicIndexRoot, TopicIndex
import DLGlobals
from DocumentStore import DocumentStore
from TopicIndex import TopicIndexRoot

manage_addDocumentLibraryForm = DTMLFile('dtml/AddLibraryForm', globals())

def manage_addDocumentLibrary(self, id, title='', REQUEST=None):
    """Construct a Document Library object"""
    
    if REQUEST is not None: 
        file = REQUEST.get('file', None)
    else:
        file = None
                                                                
    self._setObject(id, DocumentLibrary(id, title))
    
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)
                    
                                                                                                
class DocumentLibrary(Folder):
    """Document Library Zope Product"""

    meta_type = 'Document Library'

    #manage_options = (
    #    {'label': 'Contents', 'action': 'manage_main'},
    #)                
    
    _properties = (
        { 'id': 'title', 'type': 'string', 'mode': 'w' },
        { 'id': 'document_types', 'type': 'lines', 'mode': 'wd' },
        { 'id': 'deny_content_types', 'type': 'lines', 'mode': 'wd' }
    )

    # Export the special values to the namespace for better portability/readability
    document_unreviewed_date = DLGlobals.document_unreviewed_date
    document_rejected_date = DLGlobals.document_rejected_date
    document_min_reviewed_date = DLGlobals.document_min_reviewed_date
    document_no_creation_date = DLGlobals.document_no_creation_date

    def __init__(self, id, title=''):
        self.id = id
        self.title = title
        self.document_types = ['None Defined']

        # MIME types not allowed to be uploaded. Here we are blanket restricting all
        # executable and unknown types by default.
        self.deny_content_types = ['application/octet-stream']
        
        # Add a global vocabulary
        vocab_id = 'Vocabulary'
        ob = Vocabulary(vocab_id, '', globbing=1)
        self._setObject(vocab_id, ob)
        
        # Add the document storage
        ob = DocumentStore('Documents', '', vocab_id, container=self)
        self._setObject('Documents', ob)

        # Add the index topic root
        ob = TopicIndexRoot('Index', 'Top of Index', vocab_id, container=self)
        self._setObject('Index', ob)

        # Add the DTML instance methods
        path = os.path.join(Globals.package_home(globals()), 'instance', 'methods')
        for id in os.listdir(path):
            file = open(os.path.join(path, id))
            ob = DTMLMethod(file.read(), __name__=id)
            self._setObject(id, ob) 

    def setFlag(self, flag_collection, flag_name, REQUEST=None):
        """Sets a flag on REQUEST in the specified flag_collection, creating the
           collection if it doesn't exist"""

        if REQUEST is None:
            REQUEST = self.REQUEST

        flags = REQUEST.get(flag_collection, {})
        flags[flag_name] = 1
        REQUEST.set(flag_collection, flags)

    def clearFlag(self, flag_collection, flag_name, REQUEST=None):
        """Clears a flag on REQUEST in the specified flag_collection, creating the
           collection if it doesn't exist"""

        if REQUEST is None:
            REQUEST = self.REQUEST

        flags = REQUEST.get(flag_collection, {})
        flags[flag_name] = 0
        REQUEST.set(flag_collection, flags)
        
    def getFlag(self, flag_collection, flag_name, REQUEST=None):
        """Returns the value for a flag of the specified flag_collection. If the
           flag does not exist, false is returned"""
            
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        flags = REQUEST.get(flag_collection, {})
        return flags.has_key(flag_name) and flags[flag_name]

Globals.InitializeClass(DocumentLibrary)
